package np.something.controllers;

import javax.validation.Valid;
import np.something.DTO.CommentDto;
import np.something.WebConfiguration;
import np.something.model.Comment;
import np.something.services.*;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/comment")
public class CommentController {
    private final CommentService commentService;
    private final CustomerService customerService;
    private final PostService postService;

    public CommentController(CommentService commentService, CustomerService customerService, PostService postService) {
        this.commentService = commentService;
        this.customerService = customerService;
        this.postService = postService;
    }

    @GetMapping("/{id}")
    public CommentDto getComment(@PathVariable Long id) {
        return new CommentDto(commentService.findComment(id));
    }

    @GetMapping
    public List<CommentDto> getComments() {
        return commentService.findAllComments().stream()
                .map(CommentDto::new)
                .toList();
    }

    @PostMapping
    public CommentDto createComment(@RequestBody @Valid CommentDto commentDto){
        final Comment comment = commentService.addComment(
                customerService.findCustomer(commentDto.getCustomerId()),
                postService.findPost(commentDto.getPostId()),
                commentDto.getContent()
        );
        return new CommentDto(comment);
    }

    @PutMapping("/{id}")
    public CommentDto updateComment(@RequestBody @Valid CommentDto commentDto, @PathVariable Long id) {
        return new CommentDto(commentService.updateComment(id, commentDto.getContent()));
    }

    @DeleteMapping("/{id}")
    public CommentDto deleteComment(@PathVariable Long id) {
        return new CommentDto(commentService.deleteComment(id));
    }

    @DeleteMapping
    public void deleteAllComments(){
        commentService.deleteAllComments();
    }
}
