package np.something.controllers;

import javax.validation.Valid;
import np.something.DTO.CustomerDto;
import np.something.WebConfiguration;
import np.something.model.Customer;
import np.something.services.*;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/customer")
public class CustomerController {
    public static final String URL_LOGIN = "/jwt/login";
    private final CustomerService customerService;

    public CustomerController(CustomerService customerService) {
        this.customerService = customerService;
    }

    @GetMapping("/{id}")
    public CustomerDto getCustomer(@PathVariable Long id) {
        return new CustomerDto(customerService.findCustomer(id));
    }

    @GetMapping
    public List<CustomerDto> getCustomers() {
        return customerService.findAllCustomers().stream()
                .map(CustomerDto::new)
                .toList();
    }

    @PostMapping
    public CustomerDto createCustomer(@RequestBody @Valid CustomerDto customerDto){
        final Customer customer = customerService.addCustomer(customerDto.getUsername(), customerDto.getPassword());
        return new CustomerDto(customer);
    }

    @PutMapping("/{id}")
    public CustomerDto updateCustomer(@RequestBody @Valid CustomerDto customerDto, @PathVariable Long id) {
        return new CustomerDto(customerService.updateCustomer(id, customerDto.getUsername(), customerDto.getPassword()));
    }

    @DeleteMapping("/{id}")
    public CustomerDto deleteCustomer(@PathVariable Long id) {
        return new CustomerDto(customerService.deleteCustomer(id));
    }

    @DeleteMapping
    public void deleteAllCustomers(){
        customerService.deleteAllCustomers();
    }

    @GetMapping("/find/{username}")
    public CustomerDto getCustomerByUsername(@PathVariable String username) {
        return new CustomerDto(customerService.findByUsername(username));
    }

    @PostMapping(URL_LOGIN)
    public String login(@RequestBody @Valid CustomerDto customerDto) {
        return customerService.loginAndGetToken(customerDto);
    }

    @GetMapping ("/me")
    CustomerDto getCurrentCustomer(@RequestHeader(HttpHeaders.AUTHORIZATION) String token) {
        return new CustomerDto(customerService.findByUsername(customerService.loadUserByToken(token.substring(7)).getUsername()));
    }

    @GetMapping("role/{token}")
    public String getRoleByToken(@PathVariable String token) {
        var userDetails = customerService.loadUserByToken(token);
        Customer customer = customerService.findByUsername(userDetails.getUsername());

        if (customer != null) {
            return customer.getRole().toString();
        }

        return null;
    }
}
