package np.something.controllers;

import javax.validation.Valid;
import np.something.DTO.PostDto;
import np.something.WebConfiguration;
import np.something.services.CustomerService;
import np.something.services.PostService;
import org.springframework.web.bind.annotation.*;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/post")
public class PostController {
    private final PostService postService;
    private final CustomerService customerService;

    public PostController(PostService postService, CustomerService customerService) {
        this.postService = postService;
        this.customerService = customerService;
    }

    @GetMapping("/{id}")
    public PostDto getPost(@PathVariable Long id) {
        return new PostDto(postService.findPost(id));
    }

    @GetMapping
    public List<PostDto> getPosts() {
        return postService.findAllPosts().stream()
                .map(PostDto::new)
                .toList();
    }

    @PostMapping
    public PostDto createPost(@RequestBody @Valid PostDto postDto)
    {
        return new PostDto(postService.addPost(customerService.findCustomer(postDto.getCustomerId()), postDto.getTitle(), postDto.getContent()));
    }

    @PutMapping("/{id}")
    public PostDto updatePost(@RequestBody @Valid PostDto postDto, @PathVariable Long id)
    {
        return new PostDto(postService.updatePost(id, postDto.title, postDto.content));
    }

    @DeleteMapping("/{id}")
    public PostDto deletePost (@PathVariable Long id) {
        return  new PostDto(postService.deletePost(id));
    }

    @DeleteMapping
    public void deleteAllPosts() {
        postService.deleteAllPosts();
    }

    @GetMapping("/search")
    public List<PostDto> searchPosts(@RequestParam(required = false) String query) {
        if (query == null || query.isBlank()) {
            return postService.findAllPosts().stream()
                    .map(PostDto::new)
                    .toList();
        } else {
            return postService.searchPosts(query);
        }
    }
}