package np.something.model;

import org.h2.engine.User;

import javax.persistence.*;
import javax.validation.constraints.NotBlank;

import java.util.*;

@Entity
public class Customer {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    @Column
    @NotBlank(message = "Username cannot be empty")
    private String username;
    @Column
    @NotBlank(message = "Password cannot be empty")
    private String password;
    @OneToMany(fetch = FetchType.EAGER, mappedBy = "customer", cascade = CascadeType.ALL)
    private List<Comment> comments;
    @OneToMany(fetch = FetchType.EAGER, mappedBy = "customer", cascade = CascadeType.ALL)
    private List<Post> posts;

    private UserRole role;

    public Customer() {

    }

    public Customer(String username, String password) {
        this.username = username;
        this.password = password;
        this.role = UserRole.USER;
        this.comments = new ArrayList<>();
        this.posts = new ArrayList<>();
    }

    public Customer(String username, String password, UserRole role) {
        this.username = username;
        this.password = password;
        this.role = role;
        this.comments = new ArrayList<>();
        this.posts = new ArrayList<>();
    }

    public Long getId() {
        return id;
    }

    public String getUsername() {
        return username;
    }

    public String getPassword() {
        return password;
    }

    public List<Comment> getComments() {
        return comments;
    }

    public List<Post> getPosts() {
        return posts;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public UserRole getRole() {
        return role;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null || getClass() != obj.getClass()) return false;
        Customer customer = (Customer) obj;
        return Objects.equals(id, customer.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }
}
