package np.something.mvc;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;
import np.something.DTO.CustomerDto;
import np.something.model.UserRole;
import np.something.services.CustomerService;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

@Controller
@RequestMapping("/admin")
@Secured({UserRole.AsString.ADMIN})
public class Admin {
    private final CustomerService customerService;

    public Admin(CustomerService customerService) {
        this.customerService = customerService;
    }

    @GetMapping(value = { "/", "/{id}" })
    @Secured({UserRole.AsString.ADMIN})
    public String getCustomers(@PathVariable(required = false) Long id, HttpServletRequest request, Model model) {
        model.addAttribute("request", request);
        model.addAttribute("currentCustomerId", customerService.findByUsername(
                ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername()
        ).getId());
        if (id == null || id <= 0) {
            model.addAttribute("customers", customerService.findAllCustomers().stream().map(CustomerDto::new).toList());
            return "admin";
        } else {
            return "redirect:/customers/" + id;
        }
    }

    @PostMapping("/delete/{id}")
    public String deleteCustomer(@PathVariable Long id) {
        customerService.deleteCustomer(id);
        return "redirect:/admin/";
    }

    @PostMapping(value = { "/", "/{id}"})
    public String manipulateCustomer(@PathVariable(required = false) Long id, @ModelAttribute @Valid CustomerDto customerDto,
                                     HttpServletRequest request,
                                     BindingResult bindingResult,
                                     Model model) {
        model.addAttribute("request", request);
        model.addAttribute("currentCustomerId", customerService.findByUsername(
                ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername()
        ).getId());
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "/admin";
        }

        if (id == null || id <= 0) {
            customerService.addCustomer(customerDto.username, customerDto.password);
        } else {
            customerService.updateCustomer(id, customerDto.username, customerDto.password);
        }

        return "redirect:/admin/";
    }
}
