package np.something.mvc;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;
import np.something.DTO.CommentDto;
import np.something.DTO.PostDto;
import np.something.services.CommentService;
import np.something.services.CustomerService;
import np.something.services.PostService;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

@Controller
@RequestMapping("/comments")
public class Comments {
    private final CustomerService customerService;
    private final CommentService commentService;
    private final PostService postService;

    public Comments(CustomerService customerService, CommentService commentService, PostService postService) {
        this.customerService = customerService;
        this.commentService = commentService;
        this.postService = postService;
    }

    @PostMapping(value = { "/", "/{id}"})
    public String manipulateComment(@PathVariable(required = false) Long id, @ModelAttribute @Valid CommentDto commentDto,
                                    HttpServletRequest request, BindingResult bindingResult, Model model) {
        model.addAttribute("request", request);
        model.addAttribute("currentCustomerId", customerService.findByUsername(
                ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername()
        ).getId());
        model.addAttribute("posts", postService.findAllPosts().stream().map(PostDto::new).toList());

        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "/feed";
        }

        if (id == null || id <= 0) {
            commentService.addComment(customerService.findCustomer(commentDto.customerId), postService.findPost(commentDto.postId), commentDto.content);
        } else {
            commentService.updateComment(id, commentDto.content);
        }

        return "redirect:/feed";
    }

    @PostMapping("/delete/{id}")
    public String deleteComment(@PathVariable Long id) {
        commentService.deleteComment(id);
        return "redirect:/feed";
    }
}
