package np.something.mvc;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;
import np.something.DTO.CustomerDto;
import np.something.services.CustomerService;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

@Controller
@RequestMapping("/customers")
public class Customers {
    private final CustomerService customerService;

    public Customers(CustomerService customerService) {
        this.customerService = customerService;
    }

    @GetMapping(value = { "/", "/{id}" })
    public String getCustomers(@PathVariable(required = false) Long id, HttpServletRequest request, HttpSession session, Model model) {
        model.addAttribute("request", request);
        model.addAttribute("currentCustomerId", customerService.findByUsername(
                ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername()
        ).getId());
        if (id == null || id <= 0) {
            model.addAttribute("customers", customerService.findAllCustomers().stream().map(CustomerDto::new).toList());
        } else {
            model.addAttribute("customers", new CustomerDto[] { new CustomerDto(customerService.findCustomer(id)) });
        }

        return "customers";
    }

    @PostMapping("/delete/{id}")
    public String deleteCustomer(@PathVariable Long id, HttpSession session) {
        customerService.deleteCustomer(id);
        return "redirect:/customers/";
    }

    @PostMapping(value = { "/", "/{id}"})
    public String manipulateCustomer(@PathVariable(required = false) Long id, @ModelAttribute @Valid CustomerDto customerDto,
                                     HttpServletRequest request, HttpSession session,
                                     BindingResult bindingResult,
                                     Model model) {
        model.addAttribute("request", request);
        model.addAttribute("currentCustomerId", customerService.findByUsername(
                ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername()
        ).getId());
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "/customers";
        }

        if (id == null || id <= 0) {
            customerService.addCustomer(customerDto.username, customerDto.password);
        } else {
            customerService.updateCustomer(id, customerDto.username, customerDto.password);
        }

        return "redirect:/customers/";
    }
}
