package np.something.mvc;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import np.something.DTO.CommentDto;
import np.something.DTO.CustomerDto;
import np.something.DTO.PostDto;
import np.something.model.Post;
import np.something.services.CommentService;
import np.something.services.CustomerService;
import np.something.services.PostService;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Controller
@RequestMapping(value = { "", "/feed" })
public class Feed {
    private final PostService postService;
    private final CustomerService customerService;

    private final CommentService commentService;

    public Feed(PostService postService, CustomerService customerService, CommentService commentService) {
        this.postService = postService;
        this.customerService = customerService;
        this.commentService = commentService;
    }

    @GetMapping
    public String getPosts(@RequestParam(required = false) String search, HttpServletRequest request, HttpSession session, Model model) {
        model.addAttribute("request", request);
        model.addAttribute("currentCustomerId", customerService.findByUsername(
                ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername()
        ).getId());
        if (search == null) {
            model.addAttribute("posts", postService.findAllPosts().stream().map(PostDto::new).toList());
        } else {
            model.addAttribute("posts", postService.searchPosts(search));
        }
        model.addAttribute("customers", customerService.findAllCustomers().stream().map(CustomerDto::new).toList());

        return "/feed";
    }
}
