package np.something.mvc;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import np.something.services.CommentService;
import np.something.services.CustomerService;
import np.something.services.PostService;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.BindingResult;
import javax.validation.Valid;
import np.something.DTO.PostDto;

@Controller
@RequestMapping("/posts")
public class Posts {
    private final CustomerService customerService;
    private final CommentService commentService;
    private final PostService postService;

    public Posts(CustomerService customerService, CommentService commentService, PostService postService) {
        this.customerService = customerService;
        this.commentService = commentService;
        this.postService = postService;
    }

    @PostMapping("/delete/{id}")
    public String deletePost(@PathVariable Long id) {
        postService.deletePost(id);
        return "redirect:/feed";
    }

    @PostMapping(value = { "/", "/{id}"})
    public String manipulatePost(@PathVariable(required = false) Long id, @ModelAttribute @Valid PostDto postDto,
                                     HttpServletRequest request, HttpSession session,
                                     BindingResult bindingResult,
                                     Model model) {
        model.addAttribute("request", request);
        model.addAttribute("currentCustomerId", customerService.findByUsername(
                ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername()
        ).getId());
        model.addAttribute("posts", postService.findAllPosts().stream().map(PostDto::new).toList());

        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "/feed";
        }

        if (id == null || id <= 0) {
            postService.addPost(customerService.findCustomer(postDto.customerId), postDto.title, postDto.content);
        } else {
            postService.updatePost(id, postDto.title, postDto.content);
        }

        return "redirect:/feed";
    }
}
