package np.something.mvc;

import np.something.DTO.UserSignupDto;
import np.something.model.Customer;
import np.something.services.CustomerService;
import np.something.util.validation.ValidationException;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

@Controller
@RequestMapping(UserSignUp.SIGNUP_URL)
public class UserSignUp {
    public static final String SIGNUP_URL = "/signup";

    private final CustomerService customerService;

    public UserSignUp(CustomerService customerService) {
        this.customerService = customerService;
    }

    @GetMapping
    public String showSignupForm(Model model) {
        model.addAttribute("userDto", new UserSignupDto());
        return "signup";
    }

    @PostMapping
    public String signup(@ModelAttribute("userDto") @Valid UserSignupDto userSignupDto,
                         BindingResult bindingResult, HttpServletRequest request,
                         Model model) {
        model.addAttribute("request", request);
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "signup";
        }
        try {
            final Customer customer = customerService.addCustomer(
                    userSignupDto.getUsername(), userSignupDto.getPassword());
            return "redirect:/login?created=" + customer.getUsername();
        } catch (ValidationException e) {
            model.addAttribute("errors", e.getMessage());
            return "signup";
        }
    }
}
